﻿// MultiTouchEvents by Charles Petzold, January 2010
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace MultiTouchEvents
{
    public partial class MainPage : UserControl
    {
        Dictionary<int, TextBox> touchDict = new Dictionary<int, TextBox>();

        public MainPage()
        {
            InitializeComponent();
            Touch.FrameReported += OnTouchFrameReported;
        }

        void OnTouchFrameReported(object sender, TouchFrameEventArgs args)
        {
            TouchPoint primaryTouchPoint = args.GetPrimaryTouchPoint(null);

            // Inhibit mouse promotion
            if (primaryTouchPoint != null && 
                        primaryTouchPoint.Action == TouchAction.Down)
                args.SuspendMousePromotionUntilTouchUp();

            TouchPointCollection touchPoints = args.GetTouchPoints(null);

            foreach (TouchPoint touchPoint in touchPoints)
            {
                TextBox txtbox = null;
                int id = touchPoint.TouchDevice.Id;

                // Limit touch points to 2
                if (touchDict.Count == 2 && !touchDict.ContainsKey(id))
                    continue;

                switch (touchPoint.Action)
                {
                    case TouchAction.Down:
                        txtbox = touchDict.ContainsValue(txtbox1) ? 
                                                    txtbox2 : txtbox1;
                        touchDict.Add(id, txtbox);
                        break;

                    case TouchAction.Move:
                        txtbox = touchDict[id];
                        break;
 
                    case TouchAction.Up:
                        txtbox = touchDict[id];
                        touchDict.Remove(id);
                        break;
                }

                txtbox.Text += String.Format("{0} {1} {2}\r\n", 
                    touchPoint.TouchDevice.Id, touchPoint.Action, 
                    touchPoint.Position);
                txtbox.Select(txtbox.Text.Length, 0);
            }
        }
    }
}
