﻿// PianoKey by Charles Petzold, January 2010
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Petzold.MultiTouch;

namespace Piano
{
    public partial class PianoKey : UserControl
    {
        public static readonly DependencyProperty IsMiddleCProperty =
            DependencyProperty.Register("IsMiddleC",
                typeof(bool),
                typeof(PianoKey),
                new PropertyMetadata(false));

        public event EventHandler Pressed;
        public event EventHandler Released;

        static Brush grayBrush = new SolidColorBrush(Colors.Gray);
        Brush savedBackground;
        bool isMouseCaptured;

        public PianoKey()
        {
            InitializeComponent();
            DataContext = this;

            TouchManager touchManager = new TouchManager(this);
            touchManager.TouchEnter += OnTouchEnter;
            touchManager.TouchLeave += OnTouchLeave;
        }

        public bool IsMiddleC
        {
            set { SetValue(IsMiddleCProperty, value); }
            get { return (bool)GetValue(IsMiddleCProperty); }
        }

        public int KeyCode { set; get; }

        void OnTouchEnter(object sender, TouchEventArgs args)
        {
            savedBackground = Background;
            Background = grayBrush;

            if (Pressed != null)
                Pressed(this, EventArgs.Empty);
        }

        void OnTouchLeave(object sender, TouchEventArgs args)
        {
            Background = savedBackground;

            if (Released != null)
                Released(this, EventArgs.Empty);
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs args)
        {
            savedBackground = Background;
            Background = grayBrush;

            if (Pressed != null)
                Pressed(this, EventArgs.Empty);

            CaptureMouse();
            isMouseCaptured = true;
            args.Handled = true;
            base.OnMouseLeftButtonDown(args);
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs args)
        {
            if (isMouseCaptured)
            {
                ReleaseMouseCapture();
                args.Handled = true;
            }
            base.OnMouseLeftButtonUp(args);
        }

        protected override void  OnLostMouseCapture(MouseEventArgs args)
        {
            Background = savedBackground;

            if (Released != null)
                Released(this, EventArgs.Empty);

            isMouseCaptured = false;
            base.OnLostMouseCapture(args);
        }
    }
} 
