﻿// PianoVoices by Charles Petzold, January 2010
using System;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Windows;
using Petzold.SoundSynthesis;

namespace Piano
{
    public class PianoVoices
    {
        List<FmPiano> availablePianos = new List<FmPiano>();
        Dictionary<object, FmPiano> playingPianos = new Dictionary<object, FmPiano>();

        public PianoVoices(MediaElement mediaElement, int sampleRate, int audioBufferLength, int numberOfVoices)
        {
            Mixer mixer = new Mixer();

            for (int voice = 0; voice < numberOfVoices; voice++)
            {
                FmPiano piano = new FmPiano(sampleRate);
                piano.EnvelopeCompleted += OnEnvelopeCompleted;
                mixer.Inputs.Add(piano);
                availablePianos.Add(piano);
            }

            PcmMediaStreamSource streamSource = new PcmMediaStreamSource(sampleRate, 1, audioBufferLength);
            streamSource.SampleProvider = mixer;

            mediaElement.SetSource(streamSource);
        }

        // (in the Piano program, referenceObject is a PianoKey)
        public void TriggerVoice(int keyCode, object referenceObject, double volume)
        {
            // or stop one that's been playing longest?
            if (availablePianos.Count == 0)
                return;

            FmPiano piano;

            // Don't play another piano based an already-playing piano key
            if (!playingPianos.TryGetValue(referenceObject, out piano))
            {
                piano = availablePianos[0];
                availablePianos.RemoveAt(0);

                piano.Frequency = 440 * Math.Pow(2, (keyCode - 69) / 12.0);
                playingPianos.Add(referenceObject, piano);
            }

            // But re-trigger it anyway
            piano.Attack(volume);
        }

        public void DampVoice(object referenceObject)
        {
            if (playingPianos.ContainsKey(referenceObject))
            {
                FmPiano piano = playingPianos[referenceObject];
                piano.Damp();
                playingPianos.Remove(referenceObject);
            }
        }

        void OnEnvelopeCompleted(object sender, EventArgs args)
        {
 	        FmPiano piano = sender as FmPiano;

            // Try to find piano in playingPianos and remove?
            // Well, eventually the key will be released, and it'll be removed in DampVoice.
            // But let the piano become available again.

            availablePianos.Add(piano);
        }
    }
}
