﻿// PitchPipeControl by Charles Petzold, January 2010
using System;
using System.Windows;
using System.Windows.Controls;

namespace Petzold.MultiTouch
{
    public partial class PitchPipeControl : UserControl
    {
        public static readonly DependencyProperty FrequencyProperty =
            DependencyProperty.Register("Frequency",
                typeof(double),
                typeof(PitchPipeControl),
                new PropertyMetadata(261.626, OnFrequencyChanged));

        public event DependencyPropertyChangedEventHandler FrequencyChanged;

        public PitchPipeControl()
        {
            DataContext = this;
            InitializeComponent();
        }

        public double Frequency
        {
            protected set { SetValue(FrequencyProperty, value); }
            get { return (double)GetValue(FrequencyProperty); }
        }

        void OnTouchDialValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            Frequency = 440 * Math.Pow(2, (args.NewValue / 30 - 9) / 12);
        }

        static void OnFrequencyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as PitchPipeControl).OnFrequencyChanged(args);
        }

        protected virtual void OnFrequencyChanged(DependencyPropertyChangedEventArgs args)
        {
            if (FrequencyChanged != null)
                FrequencyChanged(this, args);
        }
    }
}
