﻿// SawtoothOscillator by Charles Petzold, January 2010
using System;

namespace Petzold.SoundSynthesis
{
    public class SawtoothOscillator : ISampleProvider
    {
        int sampleRate;
        double frequency;
        double volume;
        uint phaseAngleIncrement;
        uint phaseAngle;
        int amplitude;

        public SawtoothOscillator(int sampleRate)
        {
            this.sampleRate = sampleRate;
        }

        public double Frequency     // in Hz
        { 
            set
            {
                frequency = value;
                phaseAngleIncrement = (uint)(frequency * uint.MaxValue / sampleRate);
            }

            get
            {
                return frequency;
            }
        }
        public double Volume    // in db
        {
            set
            {
                volume = value;
                amplitude = (int)Math.Pow(2, Math.Min(volume, 96) / 6);
            }
            get
            {
                return volume;
            }
        }

        // asynchronous
        public short GetNextSample()
        {
            int waveform = short.MinValue + (ushort)(phaseAngle >> 16);
            phaseAngle += phaseAngleIncrement;
            return (short)((amplitude * waveform) >> 16);
        }
    }
}
