﻿// Sinewave by Charles Petzold, January 2010
using System;

namespace Petzold.SoundSynthesis
{
    public class SineWave : ISampleProvider, IIntSampleProvider
    {
        static short[] waveform = new short[65536];
        int sampleRate;
        uint phaseAngle = 0;

        static SineWave()
        {
            // Create lookup table
            for (int i = 0; i < 65536; i++)
            {
                waveform[i] = (short) (short.MaxValue * Math.Sin(2 * Math.PI * i / ushort.MaxValue));
            }
        }

        public SineWave(int sampleRate)
        {
            this.sampleRate = sampleRate;
        }

        public IIntSampleProvider Frequency { get; set; }
        public IIntSampleProvider Amplitude { get; set; }

        // Asynchronous
        public short GetNextSample()
        {
            return (short)(GetNextIntSample() >> 16);
        }

        // Asynchronous
        public int GetNextIntSample()
        {
            if (Frequency == null || Amplitude == null)
                return 0;

            ushort wholePhaseAngle = (ushort)(phaseAngle >> 16);
            int sample;

            sample = waveform[wholePhaseAngle];
            long lSample = (long)sample * Amplitude.GetNextIntSample();
            sample = (int)(lSample >> 16);
            phaseAngle += (uint)(65536L * Frequency.GetNextIntSample() / sampleRate);

            return sample;
        }
    }
}
